@@ -31,6 +31,7 @@ pub use binview_to::{binview_to_fixed_size_list_dyn, binview_to_primitive_dyn};
 use dictionary_to::*;
 use polars_error::{PolarsResult, polars_bail, polars_ensure, polars_err};
 use polars_utils::IdxSize;
+use polars_utils::float16::pf16;
 pub use primitive_to::*;
 use temporal::utf8view_to_timestamp;
 pub use utf8_to::*;
@@ -539,6 +540,8 @@ pub fn cast(
                 Int64 => utf8view_to_primitive_dyn::<i64>(arr, to_type, options),
                 #[cfg(feature = "dtype-i128")]
                 Int128 => utf8view_to_primitive_dyn::<i128>(arr, to_type, options),
+                #[cfg(feature = "dtype-f16")]
+                Float16 => utf8view_to_primitive_dyn::<pf16>(arr, to_type, options),
                 Float32 => utf8view_to_primitive_dyn::<f32>(arr, to_type, options),
                 Float64 => utf8view_to_primitive_dyn::<f64>(arr, to_type, options),
                 Timestamp(time_unit, None) => {
@@ -575,6 +578,8 @@ pub fn cast(
             Int64 => primitive_to_boolean_dyn::<i64>(array, to_type.clone()),
             #[cfg(feature = "dtype-i128")]
             Int128 => primitive_to_boolean_dyn::<i128>(array, to_type.clone()),
+            #[cfg(feature = "dtype-f16")]
+            Float16 => primitive_to_boolean_dyn::<pf16>(array, to_type.clone()),
             Float32 => primitive_to_boolean_dyn::<f32>(array, to_type.clone()),
             Float64 => primitive_to_boolean_dyn::<f64>(array, to_type.clone()),
             #[cfg(feature = "dtype-decimal")]
@@ -596,6 +601,8 @@ pub fn cast(
             Int64 => boolean_to_primitive_dyn::<i64>(array),
             #[cfg(feature = "dtype-i128")]
             Int128 => boolean_to_primitive_dyn::<i128>(array),
+            #[cfg(feature = "dtype-f16")]
+            Float16 => boolean_to_primitive_dyn::<pf16>(array),
             Float32 => boolean_to_primitive_dyn::<f32>(array),
             Float64 => boolean_to_primitive_dyn::<f64>(array),
             Utf8View => boolean_to_utf8view_dyn(array),
@@ -670,6 +677,8 @@ pub fn cast(
             Int64 => binary_to_primitive_dyn::<i64, i64>(array, to_type, options),
             #[cfg(feature = "dtype-i128")]
             Int128 => binary_to_primitive_dyn::<i64, i128>(array, to_type, options),
+            #[cfg(feature = "dtype-f16")]
+            Float16 => binary_to_primitive_dyn::<i64, pf16>(array, to_type, options),
             Float32 => binary_to_primitive_dyn::<i64, f32>(array, to_type, options),
             Float64 => binary_to_primitive_dyn::<i64, f64>(array, to_type, options),
             Binary => {
@@ -711,6 +720,8 @@ pub fn cast(
         (UInt8, Int64) => primitive_to_primitive_dyn::<u8, i64>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (UInt8, Int128) => primitive_to_primitive_dyn::<u8, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (UInt8, Float16) => primitive_to_primitive_dyn::<u8, pf16>(array, to_type, as_options),
         (UInt8, Float32) => primitive_to_primitive_dyn::<u8, f32>(array, to_type, as_options),
         (UInt8, Float64) => primitive_to_primitive_dyn::<u8, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -727,6 +738,8 @@ pub fn cast(
         (UInt16, Int64) => primitive_to_primitive_dyn::<u16, i64>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (UInt16, Int128) => primitive_to_primitive_dyn::<u16, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (UInt16, Float16) => primitive_to_primitive_dyn::<u16, pf16>(array, to_type, as_options),
         (UInt16, Float32) => primitive_to_primitive_dyn::<u16, f32>(array, to_type, as_options),
         (UInt16, Float64) => primitive_to_primitive_dyn::<u16, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -743,6 +756,8 @@ pub fn cast(
         (UInt32, Int64) => primitive_to_primitive_dyn::<u32, i64>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (UInt32, Int128) => primitive_to_primitive_dyn::<u32, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (UInt32, Float16) => primitive_to_primitive_dyn::<u32, pf16>(array, to_type, as_options),
         (UInt32, Float32) => primitive_to_primitive_dyn::<u32, f32>(array, to_type, as_options),
         (UInt32, Float64) => primitive_to_primitive_dyn::<u32, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -759,6 +774,8 @@ pub fn cast(
         (UInt64, Int64) => primitive_to_primitive_dyn::<u64, i64>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (UInt64, Int128) => primitive_to_primitive_dyn::<u64, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (UInt64, Float16) => primitive_to_primitive_dyn::<u64, pf16>(array, to_type, as_options),
         (UInt64, Float32) => primitive_to_primitive_dyn::<u64, f32>(array, to_type, as_options),
         (UInt64, Float64) => primitive_to_primitive_dyn::<u64, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -782,6 +799,8 @@ pub fn cast(
         (UInt128, Int64) => primitive_to_primitive_dyn::<u128, i64>(array, to_type, options),
         #[cfg(all(feature = "dtype-u128", feature = "dtype-i128"))]
         (UInt128, Int128) => primitive_to_primitive_dyn::<u128, i128>(array, to_type, options),
+        #[cfg(all(feature = "dtype-u128", feature = "dtype-f16"))]
+        (UInt128, Float16) => primitive_to_primitive_dyn::<u128, pf16>(array, to_type, as_options),
         #[cfg(feature = "dtype-u128")]
         (UInt128, Float32) => primitive_to_primitive_dyn::<u128, f32>(array, to_type, as_options),
         #[cfg(feature = "dtype-u128")]
@@ -800,6 +819,8 @@ pub fn cast(
         (Int8, Int64) => primitive_to_primitive_dyn::<i8, i64>(array, to_type, as_options),
         #[cfg(feature = "dtype-i128")]
         (Int8, Int128) => primitive_to_primitive_dyn::<i8, i128>(array, to_type, as_options),
+        #[cfg(feature = "dtype-f16")]
+        (Int8, Float16) => primitive_to_primitive_dyn::<i8, pf16>(array, to_type, as_options),
         (Int8, Float32) => primitive_to_primitive_dyn::<i8, f32>(array, to_type, as_options),
         (Int8, Float64) => primitive_to_primitive_dyn::<i8, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -816,6 +837,8 @@ pub fn cast(
         (Int16, Int64) => primitive_to_primitive_dyn::<i16, i64>(array, to_type, as_options),
         #[cfg(feature = "dtype-i128")]
         (Int16, Int128) => primitive_to_primitive_dyn::<i16, i128>(array, to_type, as_options),
+        #[cfg(feature = "dtype-f16")]
+        (Int16, Float16) => primitive_to_primitive_dyn::<i16, pf16>(array, to_type, as_options),
         (Int16, Float32) => primitive_to_primitive_dyn::<i16, f32>(array, to_type, as_options),
         (Int16, Float64) => primitive_to_primitive_dyn::<i16, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -832,6 +855,8 @@ pub fn cast(
         (Int32, Int64) => primitive_to_primitive_dyn::<i32, i64>(array, to_type, as_options),
         #[cfg(feature = "dtype-i128")]
         (Int32, Int128) => primitive_to_primitive_dyn::<i32, i128>(array, to_type, as_options),
+        #[cfg(feature = "dtype-f16")]
+        (Int32, Float16) => primitive_to_primitive_dyn::<i32, pf16>(array, to_type, as_options),
         (Int32, Float32) => primitive_to_primitive_dyn::<i32, f32>(array, to_type, as_options),
         (Int32, Float64) => primitive_to_primitive_dyn::<i32, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -848,6 +873,8 @@ pub fn cast(
         (Int64, Int32) => primitive_to_primitive_dyn::<i64, i32>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (Int64, Int128) => primitive_to_primitive_dyn::<i64, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Int64, Float16) => primitive_to_primitive_dyn::<i64, pf16>(array, to_type, as_options),
         (Int64, Float32) => primitive_to_primitive_dyn::<i64, f32>(array, to_type, options),
         (Int64, Float64) => primitive_to_primitive_dyn::<i64, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
@@ -871,6 +898,8 @@ pub fn cast(
         (Int128, Int32) => primitive_to_primitive_dyn::<i128, i32>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (Int128, Int64) => primitive_to_primitive_dyn::<i128, i64>(array, to_type, options),
+        #[cfg(all(feature = "dtype-i128", feature = "dtype-f16"))]
+        (Int128, Float16) => primitive_to_primitive_dyn::<i128, pf16>(array, to_type, as_options),
         #[cfg(feature = "dtype-i128")]
         (Int128, Float32) => primitive_to_primitive_dyn::<i128, f32>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
@@ -878,10 +907,32 @@ pub fn cast(
         #[cfg(all(feature = "dtype-i128", feature = "dtype-decimal"))]
         (Int128, Decimal(p, s)) => integer_to_decimal_dyn::<i128>(array, *p, *s),
 
-        (Float16, Float32) => {
-            let from = array.as_any().downcast_ref().unwrap();
-            Ok(f16_to_f32(from).boxed())
-        },
+        #[cfg(feature = "dtype-f16")]
+        (Float16, UInt8) => primitive_to_primitive_dyn::<pf16, u8>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, UInt16) => primitive_to_primitive_dyn::<pf16, u16>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, UInt32) => primitive_to_primitive_dyn::<pf16, u32>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, UInt64) => primitive_to_primitive_dyn::<pf16, u64>(array, to_type, options),
+        #[cfg(all(feature = "dtype-f16", feature = "dtype-u128"))]
+        (Float16, UInt128) => primitive_to_primitive_dyn::<pf16, u128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, Int8) => primitive_to_primitive_dyn::<pf16, i8>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, Int16) => primitive_to_primitive_dyn::<pf16, i16>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, Int32) => primitive_to_primitive_dyn::<pf16, i32>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, Int64) => primitive_to_primitive_dyn::<pf16, i64>(array, to_type, options),
+        #[cfg(all(feature = "dtype-f16", feature = "dtype-i128"))]
+        (Float16, Int128) => primitive_to_primitive_dyn::<pf16, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, Float32) => primitive_to_primitive_dyn::<pf16, f32>(array, to_type, as_options),
+        #[cfg(feature = "dtype-f16")]
+        (Float16, Float64) => primitive_to_primitive_dyn::<pf16, f64>(array, to_type, as_options),
+        #[cfg(all(feature = "dtype-f16", feature = "dtype-decimal"))]
+        (Float16, Decimal(p, s)) => float_to_decimal_dyn::<pf16>(array, *p, *s),
 
         (Float32, UInt8) => primitive_to_primitive_dyn::<f32, u8>(array, to_type, options),
         (Float32, UInt16) => primitive_to_primitive_dyn::<f32, u16>(array, to_type, options),
@@ -895,6 +946,8 @@ pub fn cast(
         (Float32, Int64) => primitive_to_primitive_dyn::<f32, i64>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (Float32, Int128) => primitive_to_primitive_dyn::<f32, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float32, Float16) => primitive_to_primitive_dyn::<f32, pf16>(array, to_type, as_options),
         (Float32, Float64) => primitive_to_primitive_dyn::<f32, f64>(array, to_type, as_options),
         #[cfg(feature = "dtype-decimal")]
         (Float32, Decimal(p, s)) => float_to_decimal_dyn::<f32>(array, *p, *s),
@@ -911,6 +964,8 @@ pub fn cast(
         (Float64, Int64) => primitive_to_primitive_dyn::<f64, i64>(array, to_type, options),
         #[cfg(feature = "dtype-i128")]
         (Float64, Int128) => primitive_to_primitive_dyn::<f64, i128>(array, to_type, options),
+        #[cfg(feature = "dtype-f16")]
+        (Float64, Float16) => primitive_to_primitive_dyn::<f64, pf16>(array, to_type, as_options),
         (Float64, Float32) => primitive_to_primitive_dyn::<f64, f32>(array, to_type, options),
         #[cfg(feature = "dtype-decimal")]
         (Float64, Decimal(p, s)) => float_to_decimal_dyn::<f64>(array, *p, *s),
@@ -935,6 +990,8 @@ pub fn cast(
         (Decimal(_, _), Int64) => decimal_to_integer_dyn::<i64>(array),
         #[cfg(all(feature = "dtype-decimal", feature = "dtype-i128"))]
         (Decimal(_, _), Int128) => decimal_to_integer_dyn::<i128>(array),
+        #[cfg(all(feature = "dtype-decimal", feature = "dtype-f16"))]
+        (Decimal(_, _), Float16) => decimal_to_float_dyn::<pf16>(array),
         #[cfg(feature = "dtype-decimal")]
         (Decimal(_, _), Float32) => decimal_to_float_dyn::<f32>(array),
         #[cfg(feature = "dtype-decimal")]
@@ -1014,7 +1071,7 @@ fn cast_to_dictionary<K: DictionaryKey>(
 ) -> PolarsResult<Box<dyn Array>> {
     let array = cast(array, dict_value_type, options)?;
     let array = array.as_ref();
-    match *dict_value_type {
+    match dict_value_type.to_logical_type() {
         ArrowDataType::Int8 => primitive_to_dictionary_dyn::<i8, K>(array),
         ArrowDataType::Int16 => primitive_to_dictionary_dyn::<i16, K>(array),
         ArrowDataType::Int32 => primitive_to_dictionary_dyn::<i32, K>(array),
@@ -1059,6 +1116,7 @@ fn from_to_binview(
         Int32 => primitive_to_binview_dyn::<i32>(array),
         Int64 => primitive_to_binview_dyn::<i64>(array),
         Int128 => primitive_to_binview_dyn::<i128>(array),
+        Float16 => primitive_to_binview_dyn::<pf16>(array),
         Float32 => primitive_to_binview_dyn::<f32>(array),
         Float64 => primitive_to_binview_dyn::<f64>(array),
         Binary => binary_to_binview::<i32>(array.as_any().downcast_ref().unwrap()),
