@@ -4,9 +4,10 @@ use arrow::datatypes::{
     ArrowDataType, DTYPE_CATEGORICAL_LEGACY, DTYPE_CATEGORICAL_NEW, DTYPE_ENUM_VALUES_LEGACY,
     DTYPE_ENUM_VALUES_NEW, Field, IntegerType, IntervalUnit, TimeUnit,
 };
-use arrow::types::{NativeType, days_ms, i256};
+use arrow::types::{days_ms, i256};
 use ethnum::I256;
 use polars_compute::cast::CastOptionsImpl;
+use polars_utils::float16::pf16;
 use polars_utils::pl_str::PlSmallStr;
 
 use super::utils::filter::Filter;
@@ -504,41 +505,14 @@ pub fn page_iter_to_array(
         )?
         .collect_boxed(filter)?,
 
-        // Float16
-        (PhysicalType::FixedLenByteArray(2), Float32) => {
-            // @NOTE: To reduce code bloat, we just use the FixedSizeBinary decoder.
-
-            let (nested, array, ptm) = PageDecoder::new(
-                &field.name,
-                pages,
-                ArrowDataType::FixedSizeBinary(2),
-                fixed_size_binary::BinaryDecoder { size: 2 },
-                init_nested,
-            )?
-            .collect(filter)?;
-
-            let array = array
-                .into_iter()
-                .map(|mut fsb_array| {
-                    let validity = fsb_array.take_validity();
-                    let values = fsb_array.values().as_slice();
-                    assert_eq!(values.len() % 2, 0);
-                    let values = values.chunks_exact(2);
-                    let values = values
-                        .map(|v| {
-                            // SAFETY: We know that `v` is always of size two.
-                            let le_bytes: [u8; 2] = unsafe { v.try_into().unwrap_unchecked() };
-                            let v = arrow::types::f16::from_le_bytes(le_bytes);
-                            v.to_f32()
-                        })
-                        .collect();
-                    Ok(PrimitiveArray::<f32>::new(dtype.clone(), values, validity).to_boxed())
-                })
-                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
-
-            (nested, array, ptm)
-        },
-
+        (PhysicalType::FixedLenByteArray(2), Float16) => PageDecoder::new(
+            &field.name,
+            pages,
+            dtype,
+            primitive::FloatDecoder::<pf16, _, _>::unit(),
+            init_nested,
+        )?
+        .collect_boxed(filter)?,
         (PhysicalType::Float, Float32) => PageDecoder::new(
             &field.name,
             pages,
