@@ -16,5 +16,5 @@ def test_collect_all(df: pl.DataFrame, optimizations: pl.QueryOptFlags) -> None:
     lf1 = df.lazy().select(pl.col("int").sum())
     lf2 = df.lazy().select((pl.col("floats") * 2).sum())
     out = pl.collect_all([lf1, lf2], optimizations=optimizations)
-    assert cast(int, out[0].item()) == 6
-    assert cast(float, out[1].item()) == 12.0
+    assert cast("int", out[0].item()) == 6
+    assert cast("float", out[1].item()) == 12.0
